from __future__ import annotations

TODO_INCOMPLETE_ERROR_CODE = "todo_incomplete"

ASK_USER_TOOL_NAME = "ask_user"
TASK_FINISH_TOOL_NAME = "task_finish"
READ_FILE_TOOL_NAME = "read_file"
WRITE_FILE_TOOL_NAME = "write_file"
LIST_FILES_TOOL_NAME = "list_files"
FILE_STR_REPLACE_TOOL_NAME = "file_str_replace"
WORKSPACE_GREP_TOOL_NAME = "workspace_grep"
BASH_TOOL_NAME = "bash"
CHECK_BACKGROUND_COMMAND_TOOL_NAME = "check_background_command"
CREATE_SUB_TASK_TOOL_NAME = "create_sub_task"
SUB_TASK_STATUS_TOOL_NAME = "sub_task_status"
COMPRESS_MEMORY_TOOL_NAME = "compress_memory"
TODO_WRITE_TOOL_NAME = "todo_write"
READ_IMAGE_TOOL_NAME = "read_image"
FILE_INFO_TOOL_NAME = "file_info"
ACTIVATE_SKILL_TOOL_NAME = "activate_skill"
